@@ -33,11 +33,12 @@
 #include <QLibraryInfo>
 #include <QSysInfo>
 #include <QProcess>
+#include <QAtomicInt>
 
 #ifndef DISABLE_GUI
 #include "gui/guiiconprovider.h"
 #ifdef Q_OS_WIN
-#include <Windows.h>
+#include <windows.h>
 #include <QSharedMemory>
 #include <QSessionManager>
 #endif // Q_OS_WIN
@@ -48,7 +49,7 @@
 #endif // Q_OS_MAC
 #include "mainwindow.h"
 #include "addnewtorrentdialog.h"
-#include "shutdownconfirm.h"
+#include "shutdownconfirmdlg.h"
 #else // DISABLE_GUI
 #include <iostream>
 #endif // DISABLE_GUI
@@ -58,8 +59,10 @@
 #endif
 
 #include "application.h"
+#include "filelogger.h"
 #include "base/logger.h"
 #include "base/preferences.h"
+#include "base/settingsstorage.h"
 #include "base/utils/fs.h"
 #include "base/utils/misc.h"
 #include "base/iconprovider.h"
@@ -70,16 +73,34 @@
 #include "base/bittorrent/session.h"
 #include "base/bittorrent/torrenthandle.h"
 
-static const char PARAMS_SEPARATOR[] = "|";
+namespace
+{
+#define SETTINGS_KEY(name) "Application/" name
+
+    // FileLogger properties keys
+#define FILELOGGER_SETTINGS_KEY(name) SETTINGS_KEY("FileLogger/") name
+    const QString KEY_FILELOGGER_ENABLED = FILELOGGER_SETTINGS_KEY("Enabled");
+    const QString KEY_FILELOGGER_PATH = FILELOGGER_SETTINGS_KEY("Path");
+    const QString KEY_FILELOGGER_BACKUP = FILELOGGER_SETTINGS_KEY("Backup");
+    const QString KEY_FILELOGGER_DELETEOLD = FILELOGGER_SETTINGS_KEY("DeleteOld");
+    const QString KEY_FILELOGGER_MAXSIZE = FILELOGGER_SETTINGS_KEY("MaxSize");
+    const QString KEY_FILELOGGER_AGE = FILELOGGER_SETTINGS_KEY("Age");
+    const QString KEY_FILELOGGER_AGETYPE = FILELOGGER_SETTINGS_KEY("AgeType");
+
+    //just a shortcut
+    inline SettingsStorage *settings() { return  SettingsStorage::instance(); }
+
+    const QString LOG_FOLDER("logs");
+    const char PARAMS_SEPARATOR[] = "|";
+}
 
 Application::Application(const QString &id, int &argc, char **argv)
     : BaseApplication(id, argc, argv)
     , m_running(false)
-#ifndef DISABLE_GUI
-    , m_shutdownAct(ShutdownAction::None)
-#endif
+    , m_shutdownAct(ShutdownDialogAction::Exit)
 {
     Logger::initInstance();
+    SettingsStorage::initInstance();
     Preferences::initInstance();
 
 #if defined(Q_OS_MACX) && !defined(DISABLE_GUI)
@@ -92,7 +113,9 @@ Application::Application(const QString &id, int &argc, char **argv)
     setApplicationName("qBittorrent");
     initializeTranslation();
 #ifndef DISABLE_GUI
-    setStyleSheet("QStatusBar::item { border-width: 0; }");
+#ifdef QBT_USES_QT5
+    setAttribute(Qt::AA_UseHighDpiPixmaps, true);  // opt-in to the high DPI pixmap support
+#endif // QBT_USES_QT5
     setQuitOnLastWindowClosed(false);
 #ifdef Q_OS_WIN
     connect(this, SIGNAL(commitDataRequest(QSessionManager &)), this, SLOT(shutdownCleanup(QSessionManager &)), Qt::DirectConnection);
@@ -102,9 +125,112 @@ Application::Application(const QString &id, int &argc, char **argv)
     connect(this, SIGNAL(messageReceived(const QString &)), SLOT(processMessage(const QString &)));
     connect(this, SIGNAL(aboutToQuit()), SLOT(cleanup()));
 
+    if (isFileLoggerEnabled())
+        m_fileLogger = new FileLogger(fileLoggerPath(), isFileLoggerBackup(), fileLoggerMaxSize(), isFileLoggerDeleteOld(), fileLoggerAge(), static_cast<FileLogger::FileLogAgeType>(fileLoggerAgeType()));
+
     Logger::instance()->addMessage(tr("qBittorrent %1 started", "qBittorrent v3.2.0alpha started").arg(VERSION));
 }
 
+#ifndef DISABLE_GUI
+QPointer<MainWindow> Application::mainWindow()
+{
+    return m_window;
+}
+#endif
+
+bool Application::isFileLoggerEnabled() const
+{
+    return settings()->loadValue(KEY_FILELOGGER_ENABLED, true).toBool();
+}
+
+void Application::setFileLoggerEnabled(bool value)
+{
+    if (value && !m_fileLogger)
+        m_fileLogger = new FileLogger(fileLoggerPath(), isFileLoggerBackup(), fileLoggerMaxSize(), isFileLoggerDeleteOld(), fileLoggerAge(), static_cast<FileLogger::FileLogAgeType>(fileLoggerAgeType()));
+    else if (!value)
+        delete m_fileLogger;
+    settings()->storeValue(KEY_FILELOGGER_ENABLED, value);
+}
+
+QString Application::fileLoggerPath() const
+{
+    return settings()->loadValue(KEY_FILELOGGER_PATH, QVariant(Utils::Fs::QDesktopServicesDataLocation() + LOG_FOLDER)).toString();
+}
+
+void Application::setFileLoggerPath(const QString &value)
+{
+    if (m_fileLogger)
+        m_fileLogger->changePath(value);
+    settings()->storeValue(KEY_FILELOGGER_PATH, value);
+}
+
+bool Application::isFileLoggerBackup() const
+{
+    return settings()->loadValue(KEY_FILELOGGER_BACKUP, true).toBool();
+}
+
+void Application::setFileLoggerBackup(bool value)
+{
+    if (m_fileLogger)
+        m_fileLogger->setBackup(value);
+    settings()->storeValue(KEY_FILELOGGER_BACKUP, value);
+}
+
+bool Application::isFileLoggerDeleteOld() const
+{
+    return settings()->loadValue(KEY_FILELOGGER_DELETEOLD, true).toBool();
+}
+
+void Application::setFileLoggerDeleteOld(bool value)
+{
+    if (value && m_fileLogger)
+        m_fileLogger->deleteOld(fileLoggerAge(), static_cast<FileLogger::FileLogAgeType>(fileLoggerAgeType()));
+    settings()->storeValue(KEY_FILELOGGER_DELETEOLD, value);
+}
+
+int Application::fileLoggerMaxSize() const
+{
+    int val = settings()->loadValue(KEY_FILELOGGER_MAXSIZE, 10).toInt();
+    if (val < 1)
+        return 1;
+    if (val > 1000)
+        return 1000;
+    return val;
+}
+
+void Application::setFileLoggerMaxSize(const int value)
+{
+    if (m_fileLogger)
+        m_fileLogger->setMaxSize(value);
+    settings()->storeValue(KEY_FILELOGGER_MAXSIZE, std::min(std::max(value, 1), 1000));
+}
+
+int Application::fileLoggerAge() const
+{
+    int val = settings()->loadValue(KEY_FILELOGGER_AGE, 6).toInt();
+    if (val < 1)
+        return 1;
+    if (val > 365)
+        return 365;
+    return val;
+}
+
+void Application::setFileLoggerAge(const int value)
+{
+    settings()->storeValue(KEY_FILELOGGER_AGE, std::min(std::max(value, 1), 365));
+}
+
+int Application::fileLoggerAgeType() const
+{
+    int val = settings()->loadValue(KEY_FILELOGGER_AGETYPE, 1).toInt();
+    return (val < 0 || val > 2) ? 1 : val;
+}
+
+void Application::setFileLoggerAgeType(const int value)
+{
+    settings()->storeValue(KEY_FILELOGGER_AGETYPE, (value < 0 || value > 2) ? 1 : value);
+}
+
 void Application::processMessage(const QString &message)
 {
     QStringList params = message.split(QLatin1String(PARAMS_SEPARATOR), QString::SkipEmptyParts);
@@ -116,6 +242,55 @@ void Application::processMessage(const QString &message)
         m_paramsQueue.append(params);
 }
 
+void Application::runExternalProgram(BitTorrent::TorrentHandle *const torrent) const
+{
+    QString program = Preferences::instance()->getAutoRunProgram();
+    program.replace("%N", torrent->name());
+    program.replace("%L", torrent->category());
+    program.replace("%F", Utils::Fs::toNativePath(torrent->contentPath()));
+    program.replace("%R", Utils::Fs::toNativePath(torrent->rootPath()));
+    program.replace("%D", Utils::Fs::toNativePath(torrent->savePath()));
+    program.replace("%C", QString::number(torrent->filesCount()));
+    program.replace("%Z", QString::number(torrent->totalSize()));
+    program.replace("%T", torrent->currentTracker());
+    program.replace("%I", torrent->hash());
+
+    Logger *logger = Logger::instance();
+    logger->addMessage(tr("Torrent: %1, running external program, command: %2").arg(torrent->name()).arg(program));
+
+#if defined(Q_OS_UNIX)
+    QProcess::startDetached(QLatin1String("/bin/sh"), {QLatin1String("-c"), program});
+#elif defined(Q_OS_WIN)  // test cmd: `echo "%F" > "c:\ab ba.txt"`
+    static const QString cmdPath = []() -> QString {
+        WCHAR systemPath[64] = {0};
+        GetSystemDirectoryW(systemPath, sizeof(systemPath) / sizeof(WCHAR));
+        return QString::fromWCharArray(systemPath) + QLatin1String("\\cmd.exe /C ");
+    }();
+    program.prepend(QLatin1String("\"")).append(QLatin1String("\""));
+    program.prepend(cmdPath);
+    const uint cmdMaxLength = 32768;  // max length (incl. terminate char) for `lpCommandLine` in `CreateProcessW()`
+    if ((program.size() + 1) > cmdMaxLength) {
+        logger->addMessage(tr("Torrent: %1, run external program command too long (length > %2), execution failed.").arg(torrent->name()).arg(cmdMaxLength), Log::CRITICAL);
+        return;
+    }
+
+    STARTUPINFOW si = {0};
+    si.cb = sizeof(si);
+    PROCESS_INFORMATION pi = {0};
+
+    WCHAR *arg = new WCHAR[program.size() + 1];
+    program.toWCharArray(arg);
+    arg[program.size()] = L'\0';
+    if (CreateProcessW(NULL, arg, NULL, NULL, FALSE, CREATE_NO_WINDOW, NULL, NULL, &si, &pi)) {
+        CloseHandle(pi.hProcess);
+        CloseHandle(pi.hThread);
+    }
+    delete[] arg;
+#else
+    QProcess::startDetached(program);
+#endif
+}
+
 void Application::sendNotificationEmail(BitTorrent::TorrentHandle *const torrent)
 {
     // Prepare mail content
@@ -140,68 +315,58 @@ void Application::torrentFinished(BitTorrent::TorrentHandle *const torrent)
     Preferences *const pref = Preferences::instance();
 
     // AutoRun program
-    if (pref->isAutoRunEnabled()) {
-        QString program = pref->getAutoRunProgram();
-
-        program.replace("%N", torrent->name());
-        program.replace("%L", torrent->label());
-        program.replace("%F", Utils::Fs::toNativePath(torrent->contentPath()));
-        program.replace("%R", Utils::Fs::toNativePath(torrent->rootPath()));
-        program.replace("%D", Utils::Fs::toNativePath(torrent->savePath()));
-        program.replace("%C", QString::number(torrent->filesCount()));
-        program.replace("%Z", QString::number(torrent->totalSize()));
-        program.replace("%T", torrent->currentTracker());
-        program.replace("%I", torrent->hash());
-
-        QProcess::startDetached(program);
-    }
+    if (pref->isAutoRunEnabled())
+        runExternalProgram(torrent);
 
     // Mail notification
-    if (pref->isMailNotificationEnabled())
+    if (pref->isMailNotificationEnabled()) {
+        Logger::instance()->addMessage(tr("Torrent: %1, sending mail notification").arg(torrent->name()));
         sendNotificationEmail(torrent);
+    }
 }
 
 void Application::allTorrentsFinished()
 {
-#ifndef DISABLE_GUI
     Preferences *const pref = Preferences::instance();
+    bool isExit = pref->shutdownqBTWhenDownloadsComplete();
+    bool isShutdown = pref->shutdownWhenDownloadsComplete();
+    bool isSuspend = pref->suspendWhenDownloadsComplete();
+    bool isHibernate = pref->hibernateWhenDownloadsComplete();
 
-    bool will_shutdown = (pref->shutdownWhenDownloadsComplete()
-                          || pref->shutdownqBTWhenDownloadsComplete()
-                          || pref->suspendWhenDownloadsComplete()
-                          || pref->hibernateWhenDownloadsComplete());
+    bool haveAction = isExit || isShutdown || isSuspend || isHibernate;
+    if (!haveAction) return;
 
-    // Auto-Shutdown
-    if (will_shutdown) {
-        bool suspend = pref->suspendWhenDownloadsComplete();
-        bool hibernate = pref->hibernateWhenDownloadsComplete();
-        bool shutdown = pref->shutdownWhenDownloadsComplete();
-
-        // Confirm shutdown
-        ShutdownAction action = ShutdownAction::None;
-        if (suspend)
-            action = ShutdownAction::Suspend;
-        else if (hibernate)
-            action = ShutdownAction::Hibernate;
-        else if (shutdown)
-            action = ShutdownAction::Shutdown;
+    ShutdownDialogAction action = ShutdownDialogAction::Exit;
+    if (isSuspend)
+        action = ShutdownDialogAction::Suspend;
+    else if (isHibernate)
+        action = ShutdownDialogAction::Hibernate;
+    else if (isShutdown)
+        action = ShutdownDialogAction::Shutdown;
 
+#ifndef DISABLE_GUI
+    // ask confirm
+    if ((action == ShutdownDialogAction::Exit) && (pref->dontConfirmAutoExit())) {
+        // do nothing & skip confirm
+    }
+    else {
         if (!ShutdownConfirmDlg::askForConfirmation(action)) return;
-
-        // Actually shut down
-        if (suspend || hibernate || shutdown) {
-            qDebug("Preparing for auto-shutdown because all downloads are complete!");
-            // Disabling it for next time
-            pref->setShutdownWhenDownloadsComplete(false);
-            pref->setSuspendWhenDownloadsComplete(false);
-            pref->setHibernateWhenDownloadsComplete(false);
-            // Make sure preferences are synced before exiting
-            m_shutdownAct = action;
-        }
-        qDebug("Exiting the application");
-        exit();
     }
 #endif // DISABLE_GUI
+
+    // Actually shut down
+    if (action != ShutdownDialogAction::Exit) {
+        qDebug("Preparing for auto-shutdown because all downloads are complete!");
+        // Disabling it for next time
+        pref->setShutdownWhenDownloadsComplete(false);
+        pref->setSuspendWhenDownloadsComplete(false);
+        pref->setHibernateWhenDownloadsComplete(false);
+        // Make sure preferences are synced before exiting
+        m_shutdownAct = action;
+    }
+
+    qDebug("Exiting the application");
+    exit();
 }
 
 bool Application::sendParams(const QStringList &params)
@@ -225,7 +390,7 @@ void Application::processParams(const QStringList &params)
     foreach (QString param, params) {
         param = param.trimmed();
 #ifndef DISABLE_GUI
-        if (Preferences::instance()->useAdditionDialog())
+        if (AddNewTorrentDialog::isEnabled())
             AddNewTorrentDialog::show(param, m_window);
         else
 #endif
@@ -244,7 +409,7 @@ int Application::exec(const QStringList &params)
 
     BitTorrent::Session::initInstance();
     connect(BitTorrent::Session::instance(), SIGNAL(torrentFinished(BitTorrent::TorrentHandle *const)), SLOT(torrentFinished(BitTorrent::TorrentHandle *const)));
-    connect(BitTorrent::Session::instance(), SIGNAL(allTorrentsFinished()), SLOT(allTorrentsFinished()));
+    connect(BitTorrent::Session::instance(), SIGNAL(allTorrentsFinished()), SLOT(allTorrentsFinished()), Qt::QueuedConnection);
 
 #ifndef DISABLE_COUNTRIES_RESOLUTION
     Net::GeoIPManager::initInstance();
@@ -421,11 +586,9 @@ void Application::cleanup()
 #ifndef DISABLE_GUI
 #ifdef Q_OS_WIN
     // cleanup() can be called multiple times during shutdown. We only need it once.
-    static bool alreadyDone = false;
-
-    if (alreadyDone)
+    static QAtomicInt alreadyDone;
+    if (!alreadyDone.testAndSetAcquire(0, 1))
         return;
-    alreadyDone = true;
 #endif // Q_OS_WIN
 
     // Hide the window and not leave it on screen as
@@ -462,10 +625,13 @@ void Application::cleanup()
 #ifndef DISABLE_COUNTRIES_RESOLUTION
     Net::GeoIPManager::freeInstance();
 #endif
+    Net::DownloadManager::freeInstance();
     Preferences::freeInstance();
+    SettingsStorage::freeInstance();
+    delete m_fileLogger;
     Logger::freeInstance();
     IconProvider::freeInstance();
-    Net::DownloadManager::freeInstance();
+
 #ifndef DISABLE_GUI
 #ifdef Q_OS_WIN
     typedef BOOL (WINAPI *PSHUTDOWNBRDESTROY)(HWND);
@@ -475,9 +641,10 @@ void Application::cleanup()
         shutdownBRDestroy((HWND)m_window->effectiveWinId());
 #endif // Q_OS_WIN
     delete m_window;
-    if (m_shutdownAct != ShutdownAction::None) {
+#endif // DISABLE_GUI
+
+    if (m_shutdownAct != ShutdownDialogAction::Exit) {
         qDebug() << "Sending computer shutdown/suspend/hibernate signal...";
         Utils::Misc::shutdownComputer(m_shutdownAct);
     }
-#endif
 }
