@@ -41,7 +41,7 @@
 #include <QNetworkCookie>
 #include <QVariant>
 
-#include "base/types.h"
+#include "types.h"
 
 enum scheduler_days
 {
@@ -89,30 +89,23 @@ namespace DNS
     };
 }
 
+class SettingsStorage;
+
 class Preferences: public QObject
 {
     Q_OBJECT
     Q_DISABLE_COPY(Preferences)
 
-private:
     Preferences();
-    ~Preferences();
 
-    static Preferences* m_instance;
-    QHash<QString, QVariant> m_data;
-    int m_randomPort;
-    bool dirty;
-    QTimer timer;
-    mutable QReadWriteLock lock;
     const QVariant value(const QString &key, const QVariant &defaultValue = QVariant()) const;
     void setValue(const QString &key, const QVariant &value);
 
-private slots:
-    bool save();
+    static Preferences* m_instance;
+    int m_randomPort;
 
 signals:
     void changed();
-    void externalLabelAdded(QString&);
 
 public:
     static void initInstance();
@@ -122,8 +115,6 @@ public:
     // General options
     QString getLocale() const;
     void setLocale(const QString &locale);
-    bool useProgramNotification() const;
-    void useProgramNotification(bool use);
     bool deleteTorrentFilesAsDefault() const;
     void setDeleteTorrentFilesAsDefault(bool del);
     bool confirmOnExit() const;
@@ -132,6 +123,10 @@ public:
     void showSpeedInTitleBar(bool show);
     bool useAlternatingRowColors() const;
     void setAlternatingRowColors(bool b);
+    bool getHideZeroValues() const;
+    void setHideZeroValues(bool b);
+    int getHideZeroComboValues() const;
+    void setHideZeroComboValues(int n);
     bool useRandomPort() const;
     void setRandomPort(bool b);
     bool systrayIntegration() const;
@@ -154,35 +149,15 @@ public:
 #endif
 
     // Downloads
-    QString getSavePath() const;
-    void setSavePath(const QString &save_path);
-    bool isTempPathEnabled() const;
-    void setTempPathEnabled(bool enabled);
-    QString getTempPath() const;
-    void setTempPath(const QString &path);
-    QString getDefaultLabel() const;
-    void setDefaultLabel(const QString &defaultLabel);
     bool useIncompleteFilesExtension() const;
     void useIncompleteFilesExtension(bool enabled);
-    bool appendTorrentLabel() const;
-    void setAppendTorrentLabel(bool b);
     QString lastLocationPath() const;
     void setLastLocationPath(const QString &path);
     bool preAllocateAllFiles() const;
     void preAllocateAllFiles(bool enabled);
-    bool useAdditionDialog() const;
-    void useAdditionDialog(bool b);
-    bool additionDialogFront() const;
-    void additionDialogFront(bool b);
-    bool addTorrentsInPause() const;
-    void addTorrentsInPause(bool b);
-    QStringList getScanDirs() const;
-    void setScanDirs(const QStringList &dirs);
-    QList<bool> getDownloadInScanDirs() const;
-    void setDownloadInScanDirs(const QList<bool> &list);
+    QVariantHash getScanDirs() const;
+    void setScanDirs(const QVariantHash &dirs);
     QString getScanDirsLastPath() const;
-    void setScanDirsDownloadPaths(const QStringList &downloadpaths);
-    QStringList getScanDirsDownloadPaths() const;
     void setScanDirsLastPath(const QString &path);
     bool isTorrentExportEnabled() const;
     QString getTorrentExportDir() const;
@@ -281,8 +256,6 @@ public:
     void setTrackersList(const QString &val);
     qreal getGlobalMaxRatio() const;
     void setGlobalMaxRatio(qreal ratio);
-    MaxRatioAction getMaxRatioAction() const;
-    void setMaxRatioAction(MaxRatioAction act);
 
     // IP Filter
     bool isFilteringEnabled() const;
@@ -298,10 +271,6 @@ public:
     bool isSearchEnabled() const;
     void setSearchEnabled(bool enabled);
 
-    // Execution Log
-    bool isExecutionLogEnabled() const;
-    void setExecutionLogEnabled(bool b);
-
     // Queueing system
     bool isQueueingSystemEnabled() const;
     void setQueueingSystemEnabled(bool enabled);
@@ -360,6 +329,8 @@ public:
     void setHibernateWhenDownloadsComplete(bool hibernate);
     bool shutdownqBTWhenDownloadsComplete() const;
     void setShutdownqBTWhenDownloadsComplete(bool shutdown);
+    bool dontConfirmAutoExit() const;
+    void setDontConfirmAutoExit(bool dontConfirmAutoExit);
     uint diskCacheSize() const;
     void setDiskCacheSize(uint size);
     uint diskCacheTTL() const;
@@ -392,6 +363,8 @@ public:
     void setNetworkInterface(const QString& iface);
     QString getNetworkInterfaceName() const;
     void setNetworkInterfaceName(const QString& iface);
+    QString getNetworkInterfaceAddress() const;
+    void setNetworkInterfaceAddress(const QString& addr);
     bool getListenIPv6() const;
     void setListenIPv6(bool enable);
     QString getNetworkAddress() const;
@@ -406,11 +379,6 @@ public:
     bool useSystemIconTheme() const;
     void useSystemIconTheme(bool enabled);
 #endif
-    QStringList getTorrentLabels() const;
-    void setTorrentLabels(const QStringList& labels);
-    void addTorrentLabelExternal(const QString &label);
-    void addTorrentLabel(const QString& label);
-    void removeTorrentLabel(const QString& label);
     bool recursiveDownloadDisabled() const;
     void disableRecursiveDownload(bool disable = true);
 #ifdef Q_OS_WIN
@@ -421,6 +389,12 @@ public:
     static bool isMagnetLinkAssocSet();
     static void setTorrentFileAssoc(bool set);
     static void setMagnetLinkAssoc(bool set);
+#endif
+#ifdef Q_OS_MAC
+    static bool isTorrentFileAssocSet();
+    static bool isMagnetLinkAssocSet();
+    static void setTorrentFileAssoc();
+    static void setMagnetLinkAssoc();
 #endif
     bool isTrackerEnabled() const;
     void setTrackerEnabled(bool enabled);
@@ -437,19 +411,8 @@ public:
     TrayIcon::Style trayIconStyle() const;
     void setTrayIconStyle(TrayIcon::Style style);
 
-
     // Stuff that don't appear in the Options GUI but are saved
     // in the same file.
-    QByteArray getAddNewTorrentDialogState() const;
-    void setAddNewTorrentDialogState(const QByteArray &state);
-    int getAddNewTorrentDialogPos() const;
-    void setAddNewTorrentDialogPos(const int &pos);
-    int getAddNewTorrentDialogWidth() const;
-    void setAddNewTorrentDialogWidth(const int &width);
-    bool getAddNewTorrentDialogExpanded() const;
-    void setAddNewTorrentDialogExpanded(const bool expanded);
-    QStringList getAddNewTorrentDialogPathHistory() const;
-    void setAddNewTorrentDialogPathHistory(const QStringList &history);
     QDateTime getDNSLastUpd() const;
     void setDNSLastUpd(const QDateTime &date);
     QString getDNSLastIP() const;
@@ -511,7 +474,7 @@ public:
     QByteArray getTorImportGeometry() const;
     void setTorImportGeometry(const QByteArray &geometry);
     bool getStatusFilterState() const;
-    bool getLabelFilterState() const;
+    bool getCategoryFilterState() const;
     bool getTrackerFilterState() const;
     int getTransSelFilter() const;
     void setTransSelFilter(const int &index);
@@ -520,11 +483,6 @@ public:
     int getToolbarTextPosition() const;
     void setToolbarTextPosition(const int position);
 
-    // Temp code.
-    // See TorrentStatistics::loadStats() for details.
-    QVariantHash getStats() const;
-    void removeStats();
-
     //From old RssSettings class
     bool isRSSEnabled() const;
     void setRSSEnabled(const bool enabled);
@@ -538,9 +496,10 @@ public:
     void setRssFeedsUrls(const QStringList &rssFeeds);
     QStringList getRssFeedsAliases() const;
     void setRssFeedsAliases(const QStringList &rssAliases);
-    QList<QByteArray> getHostNameCookies(const QString &host_name) const;
-    QList<QNetworkCookie> getHostNameQNetworkCookies(const QString& host_name) const;
-    void setHostNameCookies(const QString &host_name, const QList<QByteArray> &cookies);
+
+    // Network
+    QList<QNetworkCookie> getNetworkCookies() const;
+    void setNetworkCookies(const QList<QNetworkCookie> &cookies);
 
     // SpeedWidget
     int getSpeedWidgetPeriod() const;
@@ -548,9 +507,11 @@ public:
     bool getSpeedWidgetGraphEnable(int id) const;
     void setSpeedWidgetGraphEnable(int id, const bool enable);
 
+    void upgrade();
+
 public slots:
     void setStatusFilterState(bool checked);
-    void setLabelFilterState(bool checked);
+    void setCategoryFilterState(bool checked);
     void setTrackerFilterState(bool checked);
 
     void apply();
