@@ -35,7 +35,7 @@
 #include <QMenu>
 #include <QFileDialog>
 
-#include "base/preferences.h"
+#include "base/settingsstorage.h"
 #include "base/net/downloadmanager.h"
 #include "base/net/downloadhandler.h"
 #include "base/bittorrent/session.h"
@@ -44,16 +44,35 @@
 #include "base/bittorrent/torrenthandle.h"
 #include "base/utils/fs.h"
 #include "base/utils/misc.h"
+#include "base/utils/string.h"
+#include "base/torrentfileguard.h"
 #include "base/unicodestrings.h"
 #include "guiiconprovider.h"
 #include "autoexpandabledialog.h"
 #include "messageboxraised.h"
-#include "ui_addnewtorrentdialog.h"
 #include "proplistdelegate.h"
 #include "torrentcontentmodel.h"
 #include "torrentcontentfiltermodel.h"
+#include "ui_addnewtorrentdialog.h"
 #include "addnewtorrentdialog.h"
 
+#define SETTINGS_KEY(name) "AddNewTorrentDialog/" name
+const QString KEY_ENABLED = SETTINGS_KEY("Enabled");
+const QString KEY_DEFAULTSAVEPATH = SETTINGS_KEY("DefaultSavePath");
+const QString KEY_DEFAULTCATEGORY = SETTINGS_KEY("DefaultCategory");
+const QString KEY_TREEHEADERSTATE = SETTINGS_KEY("TreeHeaderState");
+const QString KEY_WIDTH = SETTINGS_KEY("Width");
+const QString KEY_EXPANDED = SETTINGS_KEY("Expanded");
+const QString KEY_POSITION = SETTINGS_KEY("Position");
+const QString KEY_TOPLEVEL = SETTINGS_KEY("TopLevel");
+const QString KEY_SAVEPATHHISTORY = SETTINGS_KEY("SavePathHistory");
+
+namespace
+{
+    //just a shortcut
+    inline SettingsStorage *settings() { return  SettingsStorage::instance(); }
+}
+
 AddNewTorrentDialog::AddNewTorrentDialog(QWidget *parent)
     : QDialog(parent)
     , ui(new Ui::AddNewTorrentDialog)
@@ -67,34 +86,41 @@ AddNewTorrentDialog::AddNewTorrentDialog(QWidget *parent)
     ui->lblMetaLoading->setVisible(false);
     ui->progMetaLoading->setVisible(false);
 
-    Preferences* const pref = Preferences::instance();
-    ui->start_torrent_cb->setChecked(!pref->addTorrentsInPause());
-    ui->save_path_combo->addItem(Utils::Fs::toNativePath(pref->getSavePath()), pref->getSavePath());
-    loadSavePathHistory();
-    connect(ui->save_path_combo, SIGNAL(currentIndexChanged(int)), SLOT(onSavePathChanged(int)));
-    connect(ui->browse_button, SIGNAL(clicked()), SLOT(browseButton_clicked()));
-    ui->default_save_path_cb->setVisible(false); // Default path is selected by default
+    auto session = BitTorrent::Session::instance();
 
-    // Load labels
-    const QStringList customLabels = pref->getTorrentLabels();
-    const QString defaultLabel = pref->getDefaultLabel();
+    ui->startTorrentCheckBox->setChecked(!session->isAddTorrentPaused());
+    ui->comboTTM->blockSignals(true); //the TreeView size isn't correct if the slot does it job at this point
+    ui->comboTTM->setCurrentIndex(!session->isAutoTMMDisabledByDefault());
+    ui->comboTTM->blockSignals(false);
+    populateSavePathComboBox();
+    connect(ui->savePathComboBox, SIGNAL(currentIndexChanged(int)), SLOT(onSavePathChanged(int)));
+    connect(ui->browseButton, SIGNAL(clicked()), SLOT(browseButton_clicked()));
+    ui->defaultSavePathCheckBox->setVisible(false); // Default path is selected by default
 
-    if (!defaultLabel.isEmpty())
-        ui->label_combo->addItem(defaultLabel);
-    ui->label_combo->addItem("");
+    ui->doNotDeleteTorrentCheckBox->setVisible(TorrentFileGuard::autoDeleteMode() != TorrentFileGuard::Never);
 
-    foreach (const QString& label, customLabels)
-        if (label != defaultLabel)
-            ui->label_combo->addItem(label);
+    // Load categories
+    QStringList categories = session->categories();
+    std::sort(categories.begin(), categories.end(), Utils::String::naturalCompareCaseInsensitive);
+    QString defaultCategory = settings()->loadValue(KEY_DEFAULTCATEGORY).toString();
 
-    ui->label_combo->model()->sort(0);
-    ui->content_tree->header()->setSortIndicator(0, Qt::AscendingOrder);
+    if (!defaultCategory.isEmpty())
+        ui->categoryComboBox->addItem(defaultCategory);
+    ui->categoryComboBox->addItem("");
+
+    foreach (const QString &category, categories)
+        if (category != defaultCategory)
+            ui->categoryComboBox->addItem(category);
+
+    ui->categoryComboBox->model()->sort(0);
+    ui->contentTreeView->header()->setSortIndicator(0, Qt::AscendingOrder);
     loadState();
     // Signal / slots
     connect(ui->adv_button, SIGNAL(clicked(bool)), SLOT(showAdvancedSettings(bool)));
-    editHotkey = new QShortcut(QKeySequence("F2"), ui->content_tree, 0, 0, Qt::WidgetShortcut);
+    connect(ui->doNotDeleteTorrentCheckBox, SIGNAL(clicked(bool)), SLOT(doNotDeleteTorrentClicked(bool)));
+    editHotkey = new QShortcut(QKeySequence("F2"), ui->contentTreeView, 0, 0, Qt::WidgetShortcut);
     connect(editHotkey, SIGNAL(activated()), SLOT(renameSelectedFile()));
-    connect(ui->content_tree, SIGNAL(doubleClicked(QModelIndex)), SLOT(renameSelectedFile()));
+    connect(ui->contentTreeView, SIGNAL(doubleClicked(QModelIndex)), SLOT(renameSelectedFile()));
 
     ui->buttonBox->button(QDialogButtonBox::Ok)->setFocus();
 }
@@ -108,36 +134,49 @@ AddNewTorrentDialog::~AddNewTorrentDialog()
     delete editHotkey;
 }
 
+bool AddNewTorrentDialog::isEnabled()
+{
+    return SettingsStorage::instance()->loadValue(KEY_ENABLED, true).toBool();
+}
+
+void AddNewTorrentDialog::setEnabled(bool value)
+{
+    SettingsStorage::instance()->storeValue(KEY_ENABLED, value);
+}
+
+bool AddNewTorrentDialog::isTopLevel()
+{
+    return SettingsStorage::instance()->loadValue(KEY_TOPLEVEL, true).toBool();
+}
+
+void AddNewTorrentDialog::setTopLevel(bool value)
+{
+    SettingsStorage::instance()->storeValue(KEY_TOPLEVEL, value);
+}
+
 void AddNewTorrentDialog::loadState()
 {
-    const Preferences* const pref = Preferences::instance();
-    m_headerState = pref->getAddNewTorrentDialogState();
-    int width = pref->getAddNewTorrentDialogWidth();
+    m_headerState = settings()->loadValue(KEY_TREEHEADERSTATE).toByteArray();
+    int width = settings()->loadValue(KEY_WIDTH, -1).toInt();
     if (width >= 0) {
         QRect geo = geometry();
         geo.setWidth(width);
         setGeometry(geo);
     }
-    ui->adv_button->setChecked(pref->getAddNewTorrentDialogExpanded());
+    ui->adv_button->setChecked(settings()->loadValue(KEY_EXPANDED).toBool());
 }
 
 void AddNewTorrentDialog::saveState()
 {
-    Preferences* const pref = Preferences::instance();
     if (m_contentModel)
-        pref->setAddNewTorrentDialogState(ui->content_tree->header()->saveState());
-    pref->setAddNewTorrentDialogPos(pos().y());
-    pref->setAddNewTorrentDialogWidth(width());
-    pref->setAddNewTorrentDialogExpanded(ui->adv_button->isChecked());
+        settings()->storeValue(KEY_TREEHEADERSTATE, ui->contentTreeView->header()->saveState());
+    settings()->storeValue(KEY_POSITION, pos().y());
+    settings()->storeValue(KEY_WIDTH, width());
+    settings()->storeValue(KEY_EXPANDED, ui->adv_button->isChecked());
 }
 
 void AddNewTorrentDialog::show(QString source, QWidget *parent)
 {
-    if (source.startsWith("bc://bt/", Qt::CaseInsensitive)) {
-        qDebug("Converting bc link to magnet link");
-        source = Utils::Misc::bcLinkToMagnet(source);
-    }
-
     AddNewTorrentDialog *dlg = new AddNewTorrentDialog(parent);
 
     if (Utils::Misc::isUrl(source)) {
@@ -149,8 +188,9 @@ void AddNewTorrentDialog::show(QString source, QWidget *parent)
     }
     else {
         bool ok = false;
-        if (source.startsWith("magnet:", Qt::CaseInsensitive))
-            ok = dlg->loadMagnet(source);
+        BitTorrent::MagnetUri magnetUri(source);
+        if (magnetUri.isValid())
+            ok = dlg->loadMagnet(magnetUri);
         else
             ok = dlg->loadTorrent(source);
 
@@ -161,15 +201,21 @@ void AddNewTorrentDialog::show(QString source, QWidget *parent)
     }
 }
 
-bool AddNewTorrentDialog::loadTorrent(const QString& torrent_path)
+bool AddNewTorrentDialog::loadTorrent(const QString &torrentPath)
 {
-    if (torrent_path.startsWith("file://", Qt::CaseInsensitive))
-        m_filePath = QUrl::fromEncoded(torrent_path.toLocal8Bit()).toLocalFile();
+    if (torrentPath.startsWith("file://", Qt::CaseInsensitive))
+        m_filePath = QUrl::fromEncoded(torrentPath.toLocal8Bit()).toLocalFile();
     else
-        m_filePath = torrent_path;
+        m_filePath = torrentPath;
 
     if (!QFile::exists(m_filePath)) {
-        MessageBoxRaised::critical(0, tr("I/O Error"), tr("The torrent file does not exist."));
+        MessageBoxRaised::critical(0, tr("I/O Error"), tr("The torrent file '%1' does not exist.").arg(Utils::Fs::toNativePath(m_filePath)));
+        return false;
+    }
+
+    QFileInfo fileinfo(m_filePath);
+    if (!fileinfo.isReadable()) {
+        MessageBoxRaised::critical(0, tr("I/O Error"), tr("The torrent file '%1' cannot be read from the disk. Probably you don't have enough permissions.").arg(Utils::Fs::toNativePath(m_filePath)));
         return false;
     }
 
@@ -177,19 +223,25 @@ bool AddNewTorrentDialog::loadTorrent(const QString& torrent_path)
     QString error;
     m_torrentInfo = BitTorrent::TorrentInfo::loadFromFile(m_filePath, error);
     if (!m_torrentInfo.isValid()) {
-        MessageBoxRaised::critical(0, tr("Invalid torrent"), tr("Failed to load the torrent: %1").arg(error));
+        MessageBoxRaised::critical(0, tr("Invalid torrent"), tr("Failed to load the torrent: %1.\nError: %2", "Don't remove the '\n' characters. They insert a newline.").arg(Utils::Fs::toNativePath(m_filePath)).arg(error));
         return false;
     }
 
+    m_torrentGuard.reset(new TorrentFileGuard(m_filePath));
     m_hash = m_torrentInfo.hash();
 
     // Prevent showing the dialog if download is already present
     if (BitTorrent::Session::instance()->isKnownTorrent(m_hash)) {
         BitTorrent::TorrentHandle *const torrent = BitTorrent::Session::instance()->findTorrent(m_hash);
         if (torrent) {
-            torrent->addTrackers(m_torrentInfo.trackers());
-            torrent->addUrlSeeds(m_torrentInfo.urlSeeds());
-            MessageBoxRaised::information(0, tr("Already in download list"), tr("Torrent is already in download list. Trackers were merged."), QMessageBox::Ok);
+            if (torrent->isPrivate() || m_torrentInfo.isPrivate()) {
+                MessageBoxRaised::critical(0, tr("Already in download list"), tr("Torrent is already in download list. Trackers weren't merged because it is a private torrent."), QMessageBox::Ok);
+            }
+            else {
+                torrent->addTrackers(m_torrentInfo.trackers());
+                torrent->addUrlSeeds(m_torrentInfo.urlSeeds());
+                MessageBoxRaised::information(0, tr("Already in download list"), tr("Torrent is already in download list. Trackers were merged."), QMessageBox::Ok);
+            }
         }
         else {
             MessageBoxRaised::critical(0, tr("Cannot add torrent"), tr("Cannot add this torrent. Perhaps it is already in adding state."), QMessageBox::Ok);
@@ -199,25 +251,31 @@ bool AddNewTorrentDialog::loadTorrent(const QString& torrent_path)
 
     ui->lblhash->setText(m_hash);
     setupTreeview();
+    TMMChanged(ui->comboTTM->currentIndex());
     return true;
 }
 
-bool AddNewTorrentDialog::loadMagnet(const QString &magnet_uri)
+bool AddNewTorrentDialog::loadMagnet(const BitTorrent::MagnetUri &magnetUri)
 {
-    BitTorrent::MagnetUri magnet(magnet_uri);
-    if (!magnet.isValid()) {
+    if (!magnetUri.isValid()) {
         MessageBoxRaised::critical(0, tr("Invalid magnet link"), tr("This magnet link was not recognized"));
         return false;
     }
 
-    m_hash = magnet.hash();
+    m_torrentGuard.reset(new TorrentFileGuard(QString()));
+    m_hash = magnetUri.hash();
     // Prevent showing the dialog if download is already present
     if (BitTorrent::Session::instance()->isKnownTorrent(m_hash)) {
         BitTorrent::TorrentHandle *const torrent = BitTorrent::Session::instance()->findTorrent(m_hash);
         if (torrent) {
-            torrent->addTrackers(magnet.trackers());
-            torrent->addUrlSeeds(magnet.urlSeeds());
-            MessageBoxRaised::information(0, tr("Already in download list"), tr("Magnet link is already in download list. Trackers were merged."), QMessageBox::Ok);
+            if (torrent->isPrivate()) {
+                MessageBoxRaised::critical(0, tr("Already in download list"), tr("Torrent is already in download list. Trackers weren't merged because it is a private torrent."), QMessageBox::Ok);
+            }
+            else {
+                torrent->addTrackers(magnetUri.trackers());
+                torrent->addUrlSeeds(magnetUri.urlSeeds());
+                MessageBoxRaised::information(0, tr("Already in download list"), tr("Magnet link is already in download list. Trackers were merged."), QMessageBox::Ok);
+            }
         }
         else {
             MessageBoxRaised::critical(0, tr("Cannot add torrent"), tr("Cannot add this torrent. Perhaps it is already in adding."), QMessageBox::Ok);
@@ -228,14 +286,15 @@ bool AddNewTorrentDialog::loadMagnet(const QString &magnet_uri)
     connect(BitTorrent::Session::instance(), SIGNAL(metadataLoaded(BitTorrent::TorrentInfo)), SLOT(updateMetadata(BitTorrent::TorrentInfo)));
 
     // Set dialog title
-    QString torrent_name = magnet.name();
+    QString torrent_name = magnetUri.name();
     setWindowTitle(torrent_name.isEmpty() ? tr("Magnet link") : torrent_name);
 
     setupTreeview();
+    TMMChanged(ui->comboTTM->currentIndex());
     // Set dialog position
     setdialogPosition();
 
-    BitTorrent::Session::instance()->loadMetadata(magnet_uri);
+    BitTorrent::Session::instance()->loadMetadata(magnetUri);
     setMetadataProgressIndicator(true, tr("Retrieving metadata..."));
     ui->lblhash->setText(m_hash);
 
@@ -245,9 +304,8 @@ bool AddNewTorrentDialog::loadMagnet(const QString &magnet_uri)
 void AddNewTorrentDialog::showEvent(QShowEvent *event)
 {
     QDialog::showEvent(event);
-    Preferences* const pref = Preferences::instance();
-    if (!pref->additionDialogFront())
-        return;
+    if (!isTopLevel()) return;
+
     activateWindow();
     raise();
 }
@@ -255,47 +313,41 @@ void AddNewTorrentDialog::showEvent(QShowEvent *event)
 
 void AddNewTorrentDialog::showAdvancedSettings(bool show)
 {
+    const int minimumW = minimumWidth();
+    setMinimumWidth(width());  // to remain the same width
     if (show) {
         ui->adv_button->setText(QString::fromUtf8(C_UP));
         ui->settings_group->setVisible(true);
-        ui->info_group->setVisible(true);
-        if (m_hasMetadata && (m_torrentInfo.filesCount() > 1)) {
-            ui->content_tree->setVisible(true);
-            setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
-        }
-        else {
-            ui->content_tree->setVisible(false);
-            setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
-        }
+        ui->infoGroup->setVisible(true);
+        ui->contentTreeView->setVisible(m_hasMetadata);
         static_cast<QVBoxLayout*>(layout())->insertWidget(layout()->indexOf(ui->never_show_cb) + 1, ui->adv_button);
     }
     else {
         ui->adv_button->setText(QString::fromUtf8(C_DOWN));
         ui->settings_group->setVisible(false);
-        ui->info_group->setVisible(false);
+        ui->infoGroup->setVisible(false);
         ui->buttonsHLayout->insertWidget(0, layout()->takeAt(layout()->indexOf(ui->never_show_cb) + 1)->widget());
-        setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
     }
-    relayout();
+    adjustSize();
+    setMinimumWidth(minimumW);
 }
 
 void AddNewTorrentDialog::saveSavePathHistory() const
 {
-    QDir selected_save_path(ui->save_path_combo->itemData(ui->save_path_combo->currentIndex()).toString());
-    Preferences* const pref = Preferences::instance();
+    QDir selectedSavePath(ui->savePathComboBox->itemData(ui->savePathComboBox->currentIndex()).toString());
     // Get current history
-    QStringList history = pref->getAddNewTorrentDialogPathHistory();
-    QList<QDir> history_dirs;
+    QStringList history = settings()->loadValue(KEY_SAVEPATHHISTORY).toStringList();
+    QList<QDir> historyDirs;
     foreach(const QString dir, history)
-        history_dirs << QDir(dir);
-    if (!history_dirs.contains(selected_save_path)) {
+        historyDirs << QDir(dir);
+    if (!historyDirs.contains(selectedSavePath)) {
         // Add save path to history
-        history.push_front(selected_save_path.absolutePath());
+        history.push_front(selectedSavePath.absolutePath());
         // Limit list size
         if (history.size() > 8)
             history.pop_back();
         // Save history
-        pref->setAddNewTorrentDialogPathHistory(history);
+        settings()->storeValue(KEY_SAVEPATHHISTORY, history);
     }
 }
 
@@ -303,20 +355,12 @@ void AddNewTorrentDialog::saveSavePathHistory() const
 int AddNewTorrentDialog::indexOfSavePath(const QString &save_path)
 {
     QDir saveDir(save_path);
-    for(int i = 0; i < ui->save_path_combo->count(); ++i)
-        if (QDir(ui->save_path_combo->itemData(i).toString()) == saveDir)
+    for (int i = 0; i < ui->savePathComboBox->count(); ++i)
+        if (QDir(ui->savePathComboBox->itemData(i).toString()) == saveDir)
             return i;
     return -1;
 }
 
-void AddNewTorrentDialog::updateFileNameInSavePaths(const QString &new_filename)
-{
-    for(int i = 0; i < ui->save_path_combo->count(); ++i) {
-        const QDir folder(ui->save_path_combo->itemData(i).toString());
-        ui->save_path_combo->setItemText(i, Utils::Fs::toNativePath(folder.absoluteFilePath(new_filename)));
-    }
-}
-
 void AddNewTorrentDialog::updateDiskSpaceLabel()
 {
     // Determine torrent size
@@ -337,9 +381,9 @@ void AddNewTorrentDialog::updateDiskSpaceLabel()
 
     QString size_string = torrent_size ? Utils::Misc::friendlyUnit(torrent_size) : QString(tr("Not Available", "This size is unavailable."));
     size_string += " (";
-    size_string += tr("Free disk space: %1").arg(Utils::Misc::friendlyUnit(Utils::Fs::freeDiskSpaceOnPath(
-                                                                   ui->save_path_combo->itemData(
-                                                                       ui->save_path_combo->currentIndex()).toString())));
+    size_string += tr("Free space on disk: %1").arg(Utils::Misc::friendlyUnit(Utils::Fs::freeDiskSpaceOnPath(
+                                                                   ui->savePathComboBox->itemData(
+                                                                       ui->savePathComboBox->currentIndex()).toString())));
     size_string += ")";
     ui->size_lbl->setText(size_string);
 }
@@ -347,9 +391,10 @@ void AddNewTorrentDialog::updateDiskSpaceLabel()
 void AddNewTorrentDialog::onSavePathChanged(int index)
 {
     // Toggle default save path setting checkbox visibility
-    ui->default_save_path_cb->setChecked(false);
-    ui->default_save_path_cb->setVisible(QDir(ui->save_path_combo->itemData(ui->save_path_combo->currentIndex()).toString()) != QDir(Preferences::instance()->getSavePath()));
-    relayout();
+    ui->defaultSavePathCheckBox->setChecked(false);
+    ui->defaultSavePathCheckBox->setVisible(
+                QDir(ui->savePathComboBox->itemData(ui->savePathComboBox->currentIndex()).toString())
+                != QDir(defaultSavePath()));
 
     // Remember index
     m_oldIndex = index;
@@ -357,67 +402,54 @@ void AddNewTorrentDialog::onSavePathChanged(int index)
     updateDiskSpaceLabel();
 }
 
+void AddNewTorrentDialog::categoryChanged(int index)
+{
+    Q_UNUSED(index);
+
+    if (ui->comboTTM->currentIndex() == 1) {
+        QString savePath = BitTorrent::Session::instance()->categorySavePath(ui->categoryComboBox->currentText());
+        ui->savePathComboBox->setItemText(0, Utils::Fs::toNativePath(savePath));
+        ui->savePathComboBox->setItemData(0, savePath);
+    }
+}
+
 void AddNewTorrentDialog::browseButton_clicked()
 {
-    disconnect(ui->save_path_combo, SIGNAL(currentIndexChanged(int)), this, SLOT(onSavePathChanged(int)));
-    // User is asking for a new save path
-    QString cur_save_path = ui->save_path_combo->itemText(m_oldIndex);
-    QString new_path, old_filename, new_filename;
+    disconnect(ui->savePathComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onSavePathChanged(int)));
 
-    if (m_torrentInfo.isValid() && (m_torrentInfo.filesCount() == 1)) {
-        old_filename = Utils::Fs::fileName(cur_save_path);
-        new_path = QFileDialog::getSaveFileName(this, tr("Choose save path"), cur_save_path, QString(), 0, QFileDialog::DontConfirmOverwrite);
-        if (!new_path.isEmpty())
-            new_path = Utils::Fs::branchPath(new_path, &new_filename);
-        qDebug() << "new_path: " << new_path;
-        qDebug() << "new_filename: " << new_filename;
-    }
-    else {
-        if (!cur_save_path.isEmpty() && QDir(cur_save_path).exists())
-            new_path = QFileDialog::getExistingDirectory(this, tr("Choose save path"), cur_save_path);
-        else
-            new_path = QFileDialog::getExistingDirectory(this, tr("Choose save path"), QDir::homePath());
-    }
-    if (!new_path.isEmpty()) {
-        const int existing_index = indexOfSavePath(new_path);
-        if (existing_index >= 0) {
-            ui->save_path_combo->setCurrentIndex(existing_index);
+    // User is asking for a new save path
+    QString curSavePath = ui->savePathComboBox->itemText(m_oldIndex);
+    QString newPath;
+
+    if (!curSavePath.isEmpty() && QDir(curSavePath).exists())
+        newPath = QFileDialog::getExistingDirectory(this, tr("Choose save path"), curSavePath);
+    else
+        newPath = QFileDialog::getExistingDirectory(this, tr("Choose save path"), QDir::homePath());
+
+    if (!newPath.isEmpty()) {
+        const int existingIndex = indexOfSavePath(newPath);
+        if (existingIndex >= 0) {
+            ui->savePathComboBox->setCurrentIndex(existingIndex);
         }
         else {
             // New path, prepend to combo box
-            if (!new_filename.isEmpty())
-                ui->save_path_combo->insertItem(0, Utils::Fs::toNativePath(QDir(new_path).absoluteFilePath(new_filename)), new_path);
-            else
-                ui->save_path_combo->insertItem(0, Utils::Fs::toNativePath(new_path), new_path);
-            ui->save_path_combo->setCurrentIndex(0);
-        }
-        // Update file name in all save_paths
-        if (!new_filename.isEmpty() && !Utils::Fs::sameFileNames(old_filename, new_filename)) {
-            m_torrentInfo.renameFile(0, new_filename);
-            updateFileNameInSavePaths(new_filename);
+            ui->savePathComboBox->insertItem(0, Utils::Fs::toNativePath(newPath), newPath);
+            ui->savePathComboBox->setCurrentIndex(0);
         }
 
         onSavePathChanged(0);
     }
     else {
         // Restore index
-        ui->save_path_combo->setCurrentIndex(m_oldIndex);
+        ui->savePathComboBox->setCurrentIndex(m_oldIndex);
     }
-    connect(ui->save_path_combo, SIGNAL(currentIndexChanged(int)), SLOT(onSavePathChanged(int)));
-}
 
-void AddNewTorrentDialog::relayout()
-{
-    qApp->processEvents();
-    int min_width = minimumWidth();
-    setMinimumWidth(width());
-    adjustSize();
-    setMinimumWidth(min_width);
+    connect(ui->savePathComboBox, SIGNAL(currentIndexChanged(int)), SLOT(onSavePathChanged(int)));
 }
 
 void AddNewTorrentDialog::renameSelectedFile()
 {
-    const QModelIndexList selectedIndexes = ui->content_tree->selectionModel()->selectedRows(0);
+    const QModelIndexList selectedIndexes = ui->contentTreeView->selectionModel()->selectedRows(0);
     if (selectedIndexes.size() != 1)
         return;
     const QModelIndex &index = selectedIndexes.first();
@@ -479,6 +511,10 @@ void AddNewTorrentDialog::renameSelectedFile()
             path_items.removeLast();
             path_items << new_name_last;
             QString new_path = path_items.join("/");
+            if (Utils::Fs::sameFileNames(old_path, new_path)) {
+                qDebug("Name did not change");
+                return;
+            }
             if (!new_path.endsWith("/")) new_path += "/";
             // Check for overwriting
             for (int i = 0; i < m_torrentInfo.filesCount(); ++i) {
@@ -517,7 +553,7 @@ void AddNewTorrentDialog::setdialogPosition()
     qApp->processEvents();
     QPoint center(Utils::Misc::screenCenter(this));
     // Adjust y
-    int y = Preferences::instance()->getAddNewTorrentDialogPos();
+    int y = settings()->loadValue(KEY_POSITION, -1).toInt();
     if (y >= 0) {
         center.setY(y);
     }
@@ -529,22 +565,25 @@ void AddNewTorrentDialog::setdialogPosition()
     move(center);
 }
 
-void AddNewTorrentDialog::loadSavePathHistory()
+void AddNewTorrentDialog::populateSavePathComboBox()
 {
-    QDir default_save_path(Preferences::instance()->getSavePath());
+    QString defSavePath = defaultSavePath();
+
+    ui->savePathComboBox->clear();
+    ui->savePathComboBox->addItem(Utils::Fs::toNativePath(defSavePath), defSavePath);
+    QDir defaultSaveDir(defSavePath);
     // Load save path history
-    QStringList raw_path_history = Preferences::instance()->getAddNewTorrentDialogPathHistory();
-    foreach (const QString &sp, raw_path_history)
-        if (QDir(sp) != default_save_path)
-            ui->save_path_combo->addItem(Utils::Fs::toNativePath(sp), sp);
+    foreach (const QString &savePath, settings()->loadValue(KEY_SAVEPATHHISTORY).toStringList())
+        if (QDir(savePath) != defaultSaveDir)
+            ui->savePathComboBox->addItem(Utils::Fs::toNativePath(savePath), savePath);
 }
 
 void AddNewTorrentDialog::displayContentTreeMenu(const QPoint&)
 {
     QMenu myFilesLlistMenu;
-    const QModelIndexList selectedRows = ui->content_tree->selectionModel()->selectedRows(0);
+    const QModelIndexList selectedRows = ui->contentTreeView->selectionModel()->selectedRows(0);
     QAction *actRename = 0;
-    if ((selectedRows.size() == 1) && (m_torrentInfo.filesCount() > 1)) {
+    if (selectedRows.size() == 1) {
         actRename = myFilesLlistMenu.addAction(GuiIconProvider::instance()->getIcon("edit-rename"), tr("Rename..."));
         myFilesLlistMenu.addSeparator();
     }
@@ -584,38 +623,33 @@ void AddNewTorrentDialog::accept()
     if (!m_hasMetadata)
         disconnect(this, SLOT(updateMetadata(const BitTorrent::TorrentInfo &)));
 
-    Preferences *const pref = Preferences::instance();
     BitTorrent::AddTorrentParams params;
 
     if (ui->skip_check_cb->isChecked())
         // TODO: Check if destination actually exists
         params.skipChecking = true;
 
-    // Label
-    params.label = ui->label_combo->currentText();
+    // Category
+    params.category = ui->categoryComboBox->currentText();
 
-    if (ui->defaultLabel->isChecked())
-        pref->setDefaultLabel(params.label);
+    if (ui->defaultCategoryCheckbox->isChecked())
+        settings()->storeValue(KEY_DEFAULTCATEGORY, params.category);
 
     // Save file priorities
     if (m_contentModel)
         params.filePriorities = m_contentModel->model()->getFilePriorities();
 
-    params.addPaused = !ui->start_torrent_cb->isChecked();
+    params.addPaused = !ui->startTorrentCheckBox->isChecked();
 
-    saveSavePathHistory();
-    pref->useAdditionDialog(!ui->never_show_cb->isChecked());
+    QString savePath = ui->savePathComboBox->itemData(ui->savePathComboBox->currentIndex()).toString();
+    if (ui->comboTTM->currentIndex() != 1) { // 0 is Manual mode and 1 is Automatic mode. Handle all non 1 values as manual mode.
+        params.savePath = savePath;
+        saveSavePathHistory();
+        if (ui->defaultSavePathCheckBox->isChecked())
+            settings()->storeValue(KEY_DEFAULTSAVEPATH, savePath);
+    }
 
-    QString savePath = ui->save_path_combo->itemData(ui->save_path_combo->currentIndex()).toString();
-    if (ui->default_save_path_cb->isChecked()) {
-        pref->setSavePath(savePath);
-        pref->apply();
-    }
-    else {
-        // if we don't use default save path...
-        if (QDir(savePath) != QDir(pref->getSavePath()))
-            params.savePath = savePath;
-    }
+    setEnabled(!ui->never_show_cb->isChecked());
 
     // Add torrent
     if (!m_hasMetadata)
@@ -623,6 +657,7 @@ void AddNewTorrentDialog::accept()
     else
         BitTorrent::Session::instance()->addTorrent(m_torrentInfo, params);
 
+    m_torrentGuard->markAsAddedToSession();
     QDialog::accept();
 }
 
@@ -655,6 +690,7 @@ void AddNewTorrentDialog::updateMetadata(const BitTorrent::TorrentInfo &info)
 
     // Update UI
     setupTreeview();
+    TMMChanged(ui->comboTTM->currentIndex());
     setMetadataProgressIndicator(false, tr("Metadata retrieval complete"));
 }
 
@@ -669,7 +705,7 @@ void AddNewTorrentDialog::setMetadataProgressIndicator(bool visibleIndicator, co
 void AddNewTorrentDialog::setupTreeview()
 {
     if (!m_hasMetadata) {
-        ui->comment_lbl->setText(tr("Not Available", "This comment is unavailable"));
+        setCommentText(tr("Not Available", "This comment is unavailable"));
         ui->date_lbl->setText(tr("Not Available", "This date is unavailable"));
     }
     else {
@@ -677,42 +713,44 @@ void AddNewTorrentDialog::setupTreeview()
         setWindowTitle(m_torrentInfo.name());
 
         // Set torrent information
-        ui->comment_lbl->setText(Utils::Misc::parseHtmlLinks(m_torrentInfo.comment()));
-        ui->date_lbl->setText(!m_torrentInfo.creationDate().isNull() ? m_torrentInfo.creationDate().toString(Qt::DefaultLocaleLongDate) : tr("Not available"));
+        setCommentText(Utils::Misc::parseHtmlLinks(m_torrentInfo.comment()));
+        ui->date_lbl->setText(!m_torrentInfo.creationDate().isNull() ? m_torrentInfo.creationDate().toString(Qt::DefaultLocaleShortDate) : tr("Not available"));
 
         // Prepare content tree
-        if (m_torrentInfo.filesCount() > 1) {
-            m_contentModel = new TorrentContentFilterModel(this);
-            connect(m_contentModel->model(), SIGNAL(filteredFilesChanged()), SLOT(updateDiskSpaceLabel()));
-            ui->content_tree->setModel(m_contentModel);
-            ui->content_tree->hideColumn(PROGRESS);
-            m_contentDelegate = new PropListDelegate();
-            ui->content_tree->setItemDelegate(m_contentDelegate);
-            connect(ui->content_tree, SIGNAL(clicked(const QModelIndex &)), ui->content_tree, SLOT(edit(const QModelIndex &)));
-            connect(ui->content_tree, SIGNAL(customContextMenuRequested(const QPoint &)), this, SLOT(displayContentTreeMenu(const QPoint &)));
+        m_contentModel = new TorrentContentFilterModel(this);
+        connect(m_contentModel->model(), SIGNAL(filteredFilesChanged()), SLOT(updateDiskSpaceLabel()));
+        ui->contentTreeView->setModel(m_contentModel);
+        m_contentDelegate = new PropListDelegate();
+        ui->contentTreeView->setItemDelegate(m_contentDelegate);
+        connect(ui->contentTreeView, SIGNAL(clicked(const QModelIndex &)), ui->contentTreeView, SLOT(edit(const QModelIndex &)));
+        connect(ui->contentTreeView, SIGNAL(customContextMenuRequested(const QPoint &)), this, SLOT(displayContentTreeMenu(const QPoint &)));
 
-            // List files in torrent
-            m_contentModel->model()->setupModelData(m_torrentInfo);
-            if (!m_headerState.isEmpty())
-                ui->content_tree->header()->restoreState(m_headerState);
+        // List files in torrent
+        m_contentModel->model()->setupModelData(m_torrentInfo);
+        if (!m_headerState.isEmpty())
+            ui->contentTreeView->header()->restoreState(m_headerState);
 
-            // Expand root folder
-            ui->content_tree->setExpanded(m_contentModel->index(0, 0), true);
-        }
-        else {
-            // Update save paths (append file name to them)
-            QString single_file_relpath = m_torrentInfo.filePath(0);
-            for (int i = 0; i < ui->save_path_combo->count(); ++i)
-                ui->save_path_combo->setItemText(i, Utils::Fs::toNativePath(QDir(ui->save_path_combo->itemText(i)).absoluteFilePath(single_file_relpath)));
-        }
+        // Hide useless columns after loading the header state
+        ui->contentTreeView->hideColumn(PROGRESS);
+        ui->contentTreeView->hideColumn(REMAINING);
+
+        // Expand root folder
+        ui->contentTreeView->setExpanded(m_contentModel->index(0, 0), true);
     }
 
     updateDiskSpaceLabel();
-    showAdvancedSettings(Preferences::instance()->getAddNewTorrentDialogExpanded());
+    showAdvancedSettings(settings()->loadValue(KEY_EXPANDED, false).toBool());
     // Set dialog position
     setdialogPosition();
 }
 
+QString AddNewTorrentDialog::defaultSavePath() const
+{
+    return Utils::Fs::fromNativePath(
+                settings()->loadValue(KEY_DEFAULTSAVEPATH,
+                                      BitTorrent::Session::instance()->defaultSavePath()).toString());
+}
+
 void AddNewTorrentDialog::handleDownloadFailed(const QString &url, const QString &reason)
 {
     MessageBoxRaised::critical(0, tr("Download Error"), QString("Cannot download '%1': %2").arg(url).arg(reason));
@@ -722,7 +760,7 @@ void AddNewTorrentDialog::handleDownloadFailed(const QString &url, const QString
 void AddNewTorrentDialog::handleRedirectedToMagnet(const QString &url, const QString &magnetUri)
 {
     Q_UNUSED(url)
-    if (loadMagnet(magnetUri))
+    if (loadMagnet(BitTorrent::MagnetUri(magnetUri)))
         open();
     else
         this->deleteLater();
@@ -736,3 +774,41 @@ void AddNewTorrentDialog::handleDownloadFinished(const QString &url, const QStri
     else
         this->deleteLater();
 }
+
+void AddNewTorrentDialog::TMMChanged(int index)
+{
+    if (index != 1) { // 0 is Manual mode and 1 is Automatic mode. Handle all non 1 values as manual mode.
+        populateSavePathComboBox();
+        ui->groupBoxSavePath->setEnabled(true);
+        ui->savePathComboBox->blockSignals(false);
+        ui->savePathComboBox->setCurrentIndex(m_oldIndex < ui->savePathComboBox->count() ? m_oldIndex : ui->savePathComboBox->count() - 1);
+        ui->adv_button->setEnabled(true);
+    }
+    else {
+        ui->groupBoxSavePath->setEnabled(false);
+        ui->savePathComboBox->blockSignals(true);
+        ui->savePathComboBox->clear();
+        QString savePath = BitTorrent::Session::instance()->categorySavePath(ui->categoryComboBox->currentText());
+        ui->savePathComboBox->addItem(Utils::Fs::toNativePath(savePath), savePath);
+        ui->defaultSavePathCheckBox->setVisible(false);
+        ui->adv_button->setChecked(true);
+        ui->adv_button->setEnabled(false);
+        showAdvancedSettings(true);
+    }
+}
+
+void AddNewTorrentDialog::setCommentText(const QString &str) const
+{
+    ui->commentLabel->setText(str);
+
+    // workaround for the additional space introduced by QScrollArea
+    int lineHeight = ui->commentLabel->fontMetrics().lineSpacing();
+    int lines = 1 + str.count("\n");
+    int height = lineHeight * lines;
+    ui->scrollArea->setMaximumHeight(height);
+}
+
+void AddNewTorrentDialog::doNotDeleteTorrentClicked(bool checked)
+{
+    m_torrentGuard->setAutoRemove(!checked);
+}
