@@ -52,7 +52,7 @@
 #include "previewselect.h"
 #include "speedlimitdlg.h"
 #include "updownratiodlg.h"
-#include "options_imp.h"
+#include "optionsdlg.h"
 #include "mainwindow.h"
 #include "base/preferences.h"
 #include "torrentmodel.h"
@@ -60,6 +60,7 @@
 #include "propertieswidget.h"
 #include "guiiconprovider.h"
 #include "base/utils/fs.h"
+#include "base/utils/string.h"
 #include "autoexpandabledialog.h"
 #include "transferlistsortmodel.h"
 
@@ -310,7 +311,7 @@ void TransferListWidget::pauseVisibleTorrents()
 
 void TransferListWidget::deleteSelectedTorrents()
 {
-    if (main_window->getCurrentTabWidget() != this) return;
+    if (main_window->currentTabWidget() != this) return;
 
     const QList<BitTorrent::TorrentHandle *> torrents = getSelectedTorrents();
     if (torrents.empty()) return;
@@ -342,26 +343,26 @@ void TransferListWidget::deleteVisibleTorrents()
 void TransferListWidget::increasePrioSelectedTorrents()
 {
     qDebug() << Q_FUNC_INFO;
-    if (main_window->getCurrentTabWidget() == this)
+    if (main_window->currentTabWidget() == this)
         BitTorrent::Session::instance()->increaseTorrentsPriority(extractHashes(getSelectedTorrents()));
 }
 
 void TransferListWidget::decreasePrioSelectedTorrents()
 {
     qDebug() << Q_FUNC_INFO;
-    if (main_window->getCurrentTabWidget() == this)
+    if (main_window->currentTabWidget() == this)
         BitTorrent::Session::instance()->decreaseTorrentsPriority(extractHashes(getSelectedTorrents()));
 }
 
 void TransferListWidget::topPrioSelectedTorrents()
 {
-    if (main_window->getCurrentTabWidget() == this)
+    if (main_window->currentTabWidget() == this)
         BitTorrent::Session::instance()->topTorrentsPriority(extractHashes(getSelectedTorrents()));
 }
 
 void TransferListWidget::bottomPrioSelectedTorrents()
 {
-    if (main_window->getCurrentTabWidget() == this)
+    if (main_window->currentTabWidget() == this)
         BitTorrent::Session::instance()->bottomTorrentsPriority(extractHashes(getSelectedTorrents()));
 }
 
@@ -565,21 +566,30 @@ void TransferListWidget::toggleSelectedFirstLastPiecePrio() const
         torrent->toggleFirstLastPiecePriority();
 }
 
-void TransferListWidget::askNewLabelForSelection()
+void TransferListWidget::setSelectedAutoTMMEnabled(bool enabled) const
 {
-    // Ask for label
+    foreach (BitTorrent::TorrentHandle *const torrent, getSelectedTorrents())
+        torrent->setAutoTMMEnabled(enabled);
+}
+
+void TransferListWidget::askNewCategoryForSelection()
+{
+    // Ask for category
     bool ok;
     bool invalid;
     do {
         invalid = false;
-        const QString label = AutoExpandableDialog::getText(this, tr("New Label"), tr("Label:"), QLineEdit::Normal, "", &ok).trimmed();
-        if (ok && !label.isEmpty()) {
-            if (Utils::Fs::isValidFileSystemName(label)) {
-                setSelectionLabel(label);
+        const QString category = AutoExpandableDialog::getText(this, tr("New Category"), tr("Category:"), QLineEdit::Normal, "", &ok).trimmed();
+        if (ok && !category.isEmpty()) {
+            if (!BitTorrent::Session::isValidCategoryName(category)) {
+                QMessageBox::warning(this, tr("Invalid category name"),
+                                     tr("Category name must not contain '\\'.\n"
+                                        "Category name must not start/end with '/'.\n"
+                                        "Category name must not contain '//' sequence."));
+                invalid = true;
             }
             else {
-                QMessageBox::warning(this, tr("Invalid label name"), tr("Please don't use any special characters in the label name."));
-                invalid = true;
+                setSelectionCategory(category);
             }
         }
     } while(invalid);
@@ -605,19 +615,10 @@ void TransferListWidget::renameSelectedTorrent()
     }
 }
 
-void TransferListWidget::setSelectionLabel(QString label)
+void TransferListWidget::setSelectionCategory(QString category)
 {
     foreach (const QModelIndex &index, selectionModel()->selectedRows())
-        listModel->setData(listModel->index(mapToSource(index).row(), TorrentModel::TR_LABEL), label, Qt::DisplayRole);
-}
-
-void TransferListWidget::removeLabelFromRows(QString label)
-{
-    for (int i = 0; i < listModel->rowCount(); ++i) {
-        if (listModel->data(listModel->index(i, TorrentModel::TR_LABEL)) == label) {
-            listModel->setData(listModel->index(i, TorrentModel::TR_LABEL), "", Qt::DisplayRole);
-        }
-    }
+        listModel->setData(listModel->index(mapToSource(index).row(), TorrentModel::TR_CATEGORY), category, Qt::DisplayRole);
 }
 
 void TransferListWidget::displayListMenu(const QPoint&)
@@ -638,9 +639,9 @@ void TransferListWidget::displayListMenu(const QPoint&)
     connect(&actionPreview_file, SIGNAL(triggered()), this, SLOT(previewSelectedTorrents()));
     QAction actionSet_max_ratio(QIcon(QString::fromUtf8(":/icons/skin/ratio.png")), tr("Limit share ratio..."), 0);
     connect(&actionSet_max_ratio, SIGNAL(triggered()), this, SLOT(setMaxRatioSelectedTorrents()));
-    QAction actionSet_upload_limit(QIcon(QString::fromUtf8(":/icons/skin/seeding.png")), tr("Limit upload rate..."), 0);
+    QAction actionSet_upload_limit(QIcon(QString::fromUtf8(":/icons/skin/uploadLimit.png")), tr("Limit upload rate..."), 0);
     connect(&actionSet_upload_limit, SIGNAL(triggered()), this, SLOT(setUpLimitSelectedTorrents()));
-    QAction actionSet_download_limit(QIcon(QString::fromUtf8(":/icons/skin/download.png")), tr("Limit download rate..."), 0);
+    QAction actionSet_download_limit(QIcon(QString::fromUtf8(":/icons/skin/downloadLimit.png")), tr("Limit download rate..."), 0);
     connect(&actionSet_download_limit, SIGNAL(triggered()), this, SLOT(setDlLimitSelectedTorrents()));
     QAction actionOpen_destination_folder(GuiIconProvider::instance()->getIcon("inode-directory"), tr("Open destination folder"), 0);
     connect(&actionOpen_destination_folder, SIGNAL(triggered()), this, SLOT(openSelectedTorrentsFolder()));
@@ -668,9 +669,13 @@ void TransferListWidget::displayListMenu(const QPoint&)
     QAction actionSequential_download(tr("Download in sequential order"), 0);
     actionSequential_download.setCheckable(true);
     connect(&actionSequential_download, SIGNAL(triggered()), this, SLOT(toggleSelectedTorrentsSequentialDownload()));
-    QAction actionFirstLastPiece_prio(tr("Download first and last piece first"), 0);
+    QAction actionFirstLastPiece_prio(tr("Download first and last pieces first"), 0);
     actionFirstLastPiece_prio.setCheckable(true);
     connect(&actionFirstLastPiece_prio, SIGNAL(triggered()), this, SLOT(toggleSelectedFirstLastPiecePrio()));
+    QAction actionAutoTMM(tr("Automatic Torrent Management"), 0);
+    actionAutoTMM.setCheckable(true);
+    actionAutoTMM.setToolTip(tr("Automatic mode means that various torrent properties(eg save path) will be decided by the associated category"));
+    connect(&actionAutoTMM, SIGNAL(triggered(bool)), this, SLOT(setSelectedAutoTMMEnabled(bool)));
     // End of actions
 
     // Enable/disable pause/start action given the DL state
@@ -680,8 +685,10 @@ void TransferListWidget::displayListMenu(const QPoint&)
     bool all_same_sequential_download_mode = true, all_same_prio_firstlast = true;
     bool sequential_download_mode = false, prioritize_first_last = false;
     bool one_has_metadata = false, one_not_seed = false;
-    bool all_same_label = true;
-    QString first_label;
+    bool allSameCategory = true;
+    bool allSameAutoTMM = true;
+    bool firstAutoTMM = false;
+    QString firstCategory;
     bool first = true;
 
     BitTorrent::TorrentHandle *torrent;
@@ -692,10 +699,15 @@ void TransferListWidget::displayListMenu(const QPoint&)
         torrent = listModel->torrentHandle(mapToSource(index));
         if (!torrent) continue;
 
-        if (first_label.isEmpty() && first)
-            first_label = torrent->label();
+        if (firstCategory.isEmpty() && first)
+            firstCategory = torrent->category();
+        if (firstCategory != torrent->category())
+            allSameCategory = false;
 
-        all_same_label = (first_label == torrent->label());
+        if (first)
+            firstAutoTMM = torrent->isAutoTMMEnabled();
+        if (firstAutoTMM != torrent->isAutoTMMEnabled())
+            allSameAutoTMM = false;
 
         if (torrent->hasMetadata())
             one_has_metadata = true;
@@ -738,8 +750,8 @@ void TransferListWidget::displayListMenu(const QPoint&)
         first = false;
 
         if (one_has_metadata && one_not_seed && !all_same_sequential_download_mode
-            && !all_same_prio_firstlast && !all_same_super_seeding && !all_same_label
-            && needs_start && needs_force && needs_pause && needs_preview) {
+            && !all_same_prio_firstlast && !all_same_super_seeding && !allSameCategory
+            && needs_start && needs_force && needs_pause && needs_preview && !allSameAutoTMM) {
             break;
         }
     }
@@ -756,29 +768,35 @@ void TransferListWidget::displayListMenu(const QPoint&)
     listMenu.addAction(&actionSetTorrentPath);
     if (selectedIndexes.size() == 1)
         listMenu.addAction(&actionRename);
-    // Label Menu
-    QStringList customLabels = Preferences::instance()->getTorrentLabels();
-    customLabels.sort();
-    QList<QAction*> labelActions;
-    QMenu *labelMenu = listMenu.addMenu(GuiIconProvider::instance()->getIcon("view-categories"), tr("Label"));
-    labelActions << labelMenu->addAction(GuiIconProvider::instance()->getIcon("list-add"), tr("New...", "New label..."));
-    labelActions << labelMenu->addAction(GuiIconProvider::instance()->getIcon("edit-clear"), tr("Reset", "Reset label"));
-    labelMenu->addSeparator();
-    foreach (QString label, customLabels) {
-        label.replace('&', "&&");  // avoid '&' becomes accelerator key
-        QAction *lb = new QAction(GuiIconProvider::instance()->getIcon("inode-directory"), label, labelMenu);
-        if (all_same_label && (label == first_label)) {
-            lb->setCheckable(true);
-            lb->setChecked(true);
+    // Category Menu
+    QStringList categories = BitTorrent::Session::instance()->categories();
+    std::sort(categories.begin(), categories.end(), Utils::String::naturalCompareCaseInsensitive);
+    QList<QAction*> categoryActions;
+    QMenu *categoryMenu = listMenu.addMenu(GuiIconProvider::instance()->getIcon("view-categories"), tr("Category"));
+    categoryActions << categoryMenu->addAction(GuiIconProvider::instance()->getIcon("list-add"), tr("New...", "New category..."));
+    categoryActions << categoryMenu->addAction(GuiIconProvider::instance()->getIcon("edit-clear"), tr("Reset", "Reset category"));
+    categoryMenu->addSeparator();
+    foreach (QString category, categories) {
+        category.replace('&', "&&");  // avoid '&' becomes accelerator key
+        QAction *cat = new QAction(GuiIconProvider::instance()->getIcon("inode-directory"), category, categoryMenu);
+        if (allSameCategory && (category == firstCategory)) {
+            cat->setCheckable(true);
+            cat->setChecked(true);
         }
-        labelMenu->addAction(lb);
-        labelActions << lb;
+        categoryMenu->addAction(cat);
+        categoryActions << cat;
     }
+
+    if (allSameAutoTMM) {
+        actionAutoTMM.setChecked(firstAutoTMM);
+        listMenu.addAction(&actionAutoTMM);
+    }
+
     listMenu.addSeparator();
     if (one_not_seed)
         listMenu.addAction(&actionSet_download_limit);
-    listMenu.addAction(&actionSet_max_ratio);
     listMenu.addAction(&actionSet_upload_limit);
+    listMenu.addAction(&actionSet_max_ratio);
     if (!one_not_seed && all_same_super_seeding && one_has_metadata) {
         actionSuper_seeding_mode.setChecked(super_seeding_mode);
         listMenu.addAction(&actionSuper_seeding_mode);
@@ -801,6 +819,7 @@ void TransferListWidget::displayListMenu(const QPoint&)
             added_preview_action = true;
         }
     }
+
     if (added_preview_action)
         listMenu.addSeparator();
     if (one_has_metadata) {
@@ -823,20 +842,20 @@ void TransferListWidget::displayListMenu(const QPoint&)
     QAction *act = 0;
     act = listMenu.exec(QCursor::pos());
     if (act) {
-        // Parse label actions only (others have slots assigned)
-        int i = labelActions.indexOf(act);
+        // Parse category actions only (others have slots assigned)
+        int i = categoryActions.indexOf(act);
         if (i >= 0) {
-            // Label action
+            // Category action
             if (i == 0) {
-                // New Label
-                askNewLabelForSelection();
+                // New Category
+                askNewCategoryForSelection();
             }
             else {
-                QString label = "";
+                QString category = "";
                 if (i > 1)
-                    label = customLabels.at(i - 2);
-                // Update Label
-                setSelectionLabel(label);
+                    category = categories.at(i - 2);
+                // Update Category
+                setSelectionCategory(category);
             }
         }
     }
@@ -854,12 +873,12 @@ void TransferListWidget::currentChanged(const QModelIndex& current, const QModel
     emit currentTorrentChanged(torrent);
 }
 
-void TransferListWidget::applyLabelFilter(QString label)
+void TransferListWidget::applyCategoryFilter(QString category)
 {
-    if (label.isNull())
-        nameFilterModel->disableLabelFilter();
+    if (category.isNull())
+        nameFilterModel->disableCategoryFilter();
     else
-        nameFilterModel->setLabelFilter(label);
+        nameFilterModel->setCategoryFilter(category);
 }
 
 void TransferListWidget::applyTrackerFilterAll()
