@@ -1,4 +1,4 @@
-#VERSION: 2.17
+#VERSION: 2.20
 #AUTHORS: Diego de las Heras (ngosang@hotmail.es)
 
 # Redistribution and use in source and binary forms, with or without
@@ -36,11 +36,12 @@ class torrentz(object):
     name = 'Torrentz'
     supported_categories = {'all': ''}
 
-    trackers_list = ['udp://tracker.openbittorrent.com:80/announce',
-                    'udp://glotorrents.pw:6969/announce',
-                    'udp://tracker.leechers-paradise.org:6969',
+    trackers_list = ['udp://tracker.coppersurfer.tk:6969/announce',
+                    'udp://tracker.opentrackr.org:1337/announce',
+                    'udp://zer0day.ch:1337/announce',
+                    'udp://tracker.leechers-paradise.org:6969/announce',
                     'udp://9.rarbg.com:2710/announce',
-                    'udp://tracker.coppersurfer.tk:6969']
+                    'udp://explodie.org:6969/announce']
 
     class MyHtmlParser(HTMLParser):
         def __init__(self, results, url, trackers):
@@ -92,10 +93,9 @@ class torrentz(object):
                 # display item
                 self.td_counter = None
                 self.current_item['engine_url'] = self.url
-                if self.current_item['name'].find(' \xc2'):
-                    self.current_item['name'] = self.current_item['name'].split(' \xc2')[0]
-                self.current_item['link'] += '&' + urlencode({'dn' : self.current_item['name']})
-                self.current_item['name'] = self.current_item['name'].decode('utf8')
+                if self.current_item['name'].find(u' \xbb'):
+                    self.current_item['name'] = self.current_item['name'].split(u' \xbb')[0]
+                self.current_item['link'] += '&' + urlencode({'dn' : self.current_item['name'].encode('utf-8')})
 
                 prettyPrinter(self.current_item)
                 self.results.append('a')
