@@ -37,7 +37,9 @@ static const char *__TRANSLATIONS__[] = {
     QT_TRANSLATE_NOOP("HttpServer", "Edit"),
     QT_TRANSLATE_NOOP("HttpServer", "Help"),
     QT_TRANSLATE_NOOP("HttpServer", "Logout"),
-    QT_TRANSLATE_NOOP("HttpServer", "Download Torrents from their URL or Magnet link"),
+    QT_TRANSLATE_NOOP("HttpServer", "Exit qBittorrent"),
+    QT_TRANSLATE_NOOP("HttpServer", "Download from URLs"),
+    QT_TRANSLATE_NOOP("HttpServer", "Download Torrents from their URLs or Magnet links"),
     QT_TRANSLATE_NOOP("HttpServer", "Only one link per line"),
     QT_TRANSLATE_NOOP("HttpServer", "Upload local torrent"),
     QT_TRANSLATE_NOOP("HttpServer", "Download"),
@@ -56,8 +58,6 @@ static const char *__TRANSLATIONS__[] = {
     QT_TRANSLATE_NOOP("HttpServer", "Language"),
     QT_TRANSLATE_NOOP("HttpServer", "The port used for incoming connections must be between 1 and 65535."),
     QT_TRANSLATE_NOOP("HttpServer", "The port used for the Web UI must be between 1 and 65535."),
-    QT_TRANSLATE_NOOP("HttpServer", "The Web UI username must be at least 3 characters long."),
-    QT_TRANSLATE_NOOP("HttpServer", "The Web UI password must be at least 3 characters long."),
     QT_TRANSLATE_NOOP("HttpServer", "Save"),
     QT_TRANSLATE_NOOP("HttpServer", "qBittorrent client is not reachable"),
     QT_TRANSLATE_NOOP("HttpServer", "HTTP Server"),
@@ -75,7 +75,6 @@ static const char *__TRANSLATIONS__[] = {
     QT_TRANSLATE_NOOP("HttpServer", "Download limit:"),
     QT_TRANSLATE_NOOP("HttpServer", "Apply"),
     QT_TRANSLATE_NOOP("HttpServer", "Add"),
-    QT_TRANSLATE_NOOP("HttpServer", "Upload Torrents"),
     QT_TRANSLATE_NOOP("HttpServer", "All"),
     QT_TRANSLATE_NOOP("HttpServer", "Downloading"),
     QT_TRANSLATE_NOOP("HttpServer", "Seeding"),
@@ -83,14 +82,38 @@ static const char *__TRANSLATIONS__[] = {
     QT_TRANSLATE_NOOP("HttpServer", "Resumed"),
     QT_TRANSLATE_NOOP("HttpServer", "Paused"),
     QT_TRANSLATE_NOOP("HttpServer", "Active"),
-    QT_TRANSLATE_NOOP("HttpServer", "Inactive")
-    QT_TRANSLATE_NOOP("HttpServer", "Save files to location:")
-    QT_TRANSLATE_NOOP("HttpServer", "Label:")
-    QT_TRANSLATE_NOOP("HttpServer", "Cookie:")
+    QT_TRANSLATE_NOOP("HttpServer", "Inactive"),
+    QT_TRANSLATE_NOOP("HttpServer", "Save files to location:"),
+    QT_TRANSLATE_NOOP("HttpServer", "Category:"),
+    QT_TRANSLATE_NOOP("HttpServer", "Cookie:"),
+    QT_TRANSLATE_NOOP("HttpServer", "Type folder here"),
+    QT_TRANSLATE_NOOP("HttpServer", "Run an external program on torrent completion"),
+    QT_TRANSLATE_NOOP("HttpServer", "Enable bandwidth management (uTP)"),
+    QT_TRANSLATE_NOOP("HttpServer", "Apply rate limit to uTP connections"),
+    QT_TRANSLATE_NOOP("HttpServer", "Alternative Global Rate Limits"),
+    QT_TRANSLATE_NOOP("HttpServer", "More information"),
+    QT_TRANSLATE_NOOP("HttpServer", "Information about certificates"),
+    QT_TRANSLATE_NOOP("HttpServer", "Save Files to"),
+    QT_TRANSLATE_NOOP("HttpServer", "Watch Folder"),
+    QT_TRANSLATE_NOOP("HttpServer", "Default Folder")
 };
 
 static const struct { const char *source; const char *comment; } __COMMENTED_TRANSLATIONS__[] = {
-    QT_TRANSLATE_NOOP3("HttpServer", "Downloaded", "Is the file downloaded or not?")
+    QT_TRANSLATE_NOOP3("HttpServer", "Downloaded", "Is the file downloaded or not?"),
+    QT_TRANSLATE_NOOP3("HttpServer", "from", "from time1 to time2"),
+    QT_TRANSLATE_NOOP3("HttpServer", "to", "from time1 to time2"),
+    QT_TRANSLATE_NOOP3("HttpServer", "Other...", "Save Files to: Watch Folder / Default Folder / Other..."),
+    QT_TRANSLATE_NOOP3("HttpServer", "Every day", "Schedule the use of alternative rate limits on ..."),
+    QT_TRANSLATE_NOOP3("HttpServer", "Week days", "Schedule the use of alternative rate limits on ..."),
+    QT_TRANSLATE_NOOP3("HttpServer", "Week ends", "Schedule the use of alternative rate limits on ..."),
+    QT_TRANSLATE_NOOP3("HttpServer", "Monday", "Schedule the use of alternative rate limits on ..."),
+    QT_TRANSLATE_NOOP3("HttpServer", "Tuesday", "Schedule the use of alternative rate limits on ..."),
+    QT_TRANSLATE_NOOP3("HttpServer", "Wednesday", "Schedule the use of alternative rate limits on ..."),
+    QT_TRANSLATE_NOOP3("HttpServer", "Thursday", "Schedule the use of alternative rate limits on ..."),
+    QT_TRANSLATE_NOOP3("HttpServer", "Friday", "Schedule the use of alternative rate limits on ..."),
+    QT_TRANSLATE_NOOP3("HttpServer", "Saturday", "Schedule the use of alternative rate limits on ..."),
+    QT_TRANSLATE_NOOP3("HttpServer", "Sunday", "Schedule the use of alternative rate limits on ..."),
+    QT_TRANSLATE_NOOP3("HttpServer", "Upload Torrents", "Upload torrent files to qBittorent using WebUI")
 };
 
 #endif // EXTRA_TRANSLATIONS_H
